/***************************************************************************
                          kallelementsdlg.cpp  -  description
                             -------------------
    begin                : Die Sep 16 2003
    copyright            : (C) 2003 by Klemens Meyer
    email                : klesmen@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kallelementsdlg.h"
#include <klistview.h>
#include <klocale.h>
#include <qcolor.h>
#include <kmessagebox.h>
#include <qlabel.h>
#include <qsimplerichtext.h>
#include <qstylesheet.h>
#include <kcmenumngr.h>
#include <kqadialog.h>
#include <qhbox.h>



KElementItem::KElementItem(struct elem* elmt, KListView* parent)
	: KListViewItem(parent)
	, element(elmt)
	, changed(false)
{
	setMultiLinesEnabled(true);

	setText(0, QString::fromLocal8Bit(elmt->q));
	setText(1, QString::fromLocal8Bit(elmt->a));
}

void KElementItem::UpdateElement()
{
	if(changed)
	{
		QString	q	= text(0).local8Bit();
		QString	a	= text(1).local8Bit();

		element->q = (char*) realloc (element->q, (size_t)(q.length()+1));
		element->a = (char*) realloc (element->a, (size_t)(a.length()+1));
		strcpy (element->q, q);
		strcpy (element->a, a);

		changed	= false;
	}
}

/** Updated the changes made elsewhere in the element */
void KElementItem::UpdateItem()
{
	if(Changed())
	{
		setText(0, QString::fromLocal8Bit(element->q));
		setText(1, QString::fromLocal8Bit(element->a));
		changed	= false;
	}
}

/** Sets wether the item has changed */
bool KElementItem::Changed()
{
	return changed	=  QString::fromLocal8Bit(element->q) != text(0)
					|| QString::fromLocal8Bit(element->a) != text(1);
}






KAllElementsDlg::KAllElementsDlg(QWidget *parent, const char *name )
	: KDialogBase(parent, name, true, i18n("All Elements"))
	, changed(false)
{
	QVBox*	main	= makeVBoxMainWidget();

    QHBox*  finder  = new QHBox((QWidget*)main);

    ftext           = new KLineEdit(finder, "Findstring");
    KPushButton* f  = new KPushButton(i18n("Find element"), (QWidget*)finder, "find");

    ftext->setTrapReturnKey(true);
    finder->setSpacing(5);

//	new QLabel(i18n("Click twice on the element you want to edit, or right-click for more options.."), (QWidget*)main);
	lview			= new KListView((QWidget*)main, "klistview");
	popup			= new QPopupMenu(this, "allmenu");

	popup->insertItem(i18n("Edit"), this, SLOT(slotBigEdit()));
	popup->insertItem(i18n("Add Vice Versa too."), this, SLOT(slotViceVersa()));
	popup->insertItem(i18n("Delete"), this, SLOT(slotDelete()));

	lview->setAlternateBackground(QColor("LightCyan"));
	lview->setAllColumnsShowFocus(true);

	lview->addColumn(i18n("Question"));
	lview->addColumn(i18n("Answer"));

	lview->setColumnWidthMode(0, QListView::Maximum);
	lview->setColumnWidthMode(1, QListView::Maximum);

	lview->setItemsRenameable(true);
	lview->setRenameable(1, true);
    lview->setDefaultRenameAction(QListView::Accept);

	for(struct elem* list = ma_iterate_list(NULL) ; list != NULL ; list = ma_iterate_list(list))
	{
		(void) new KElementItem(list, lview);
	}

	connect(lview, SIGNAL(itemRenamed(QListViewItem*, const QString&, int)), this, SLOT(slotEdited(QListViewItem*, const QString&, int)));
	connect(lview, SIGNAL(contextMenu(KListView*, QListViewItem*, const QPoint&)), this, SLOT(slotPopUp(KListView*, QListViewItem*, const QPoint&)));
    connect(f, SIGNAL(clicked()), this, SLOT(slotFind()));
    connect(ftext, SIGNAL(returnPressed(const QString&)), this, SLOT(slotFTextReturn(const QString&)));
}

KAllElementsDlg::~KAllElementsDlg()
{
}


 /** The Item has changed */
 void KAllElementsDlg::slotEdited(QListViewItem* item, const QString&, int)
 {
//	KMessageBox::information(this, i18n("An item has changed!"));
	changed	= dynamic_cast<KElementItem*>(item)->Changed();
 }

/** Updates the Elements in the Listview back to the NN list. */
void KAllElementsDlg::UpdateElements()
{
	QListViewItem*	item	= lview->firstChild();
	for( ; item ; item = item->nextSibling())
	{
		dynamic_cast<KElementItem*>(item)->UpdateElement();
	}
	changed	= false;
}

/** No descriptions */
void KAllElementsDlg::slotOk()
{
	UpdateElements();
	KDialogBase::slotOk();
}

/** No descriptions */
void KAllElementsDlg::slotApply()
{
	UpdateElements();
	KDialogBase::slotApply();
}

/** No descriptions */
void KAllElementsDlg::slotCancel()
{
	if(changed)
	{
		QString	msg	= i18n("There are changed items in the list.\nDo you want to discard the changes?");
		if(KMessageBox::warningYesNo(this, msg, i18n("Cancel Show All")) == KMessageBox::No)
			return;
	}
	return KDialogBase::slotCancel();
}

/** Deletes the selected elements */
void KAllElementsDlg::slotDelete()
{
	ma_delete_el(item->getElement());
	delete item;
	item	= NULL;
}

/** Edits the element in the Edit-Elements Dialog. */
void KAllElementsDlg::slotBigEdit()
{
	KQADialog	dlg(false, i18n("Edit Element"), this, "edit", item->getElement());
	dlg.setFonts(qfont, afont);
	dlg.exec();

	if(item->Changed())
	{
		item->UpdateItem();
		changed	= true;
	}
}

/** Shows the context-menu */
void KAllElementsDlg::slotPopUp(KListView*, QListViewItem* i, const QPoint& p)
{
	item	= dynamic_cast<KElementItem*>(i);
	popup->popup(p);
}


/** Takes the Element and puts a reversed answer-question element to the learning list. */
void KAllElementsDlg::slotViceVersa()
{
	ma_make_new_el(item->getElement()->gr, item->getElement()->a, item->getElement()->q);
	elem*	element	= ma_find_el(item->getElement()->a, item->getElement()->q);
	if(element)
	{
		(void) new KElementItem(element, lview);
	}
}
/** Sets the fonts for the Big-Edit dialog. */
void KAllElementsDlg::setFonts(const QFont& q, const QFont& a)
{
	qfont	= q;
	afont	= a;
}
/** Finds an element in the list. */
void KAllElementsDlg::slotFind()
{
    QListViewItem*  fnd = lview->findItem(ftext->text(), 0, Qt::Contains);
    if(!fnd)
        fnd = lview->findItem(ftext->text(), 1, Qt::Contains);

    if(fnd)
    {
        lview->setSelected(fnd, true);
        lview->ensureItemVisible(fnd);
        lview->setFocus();
    }
    else
        KMessageBox::sorry(this, i18n("The text you entered was not found."));
}

/** called when in the edit box return is pressed. */
void KAllElementsDlg::slotFTextReturn(const QString&)
{
    slotFind();
}
