/***************************************************************************
                          kallelementsdlg.h  -  description
                             -------------------
    begin                : Die Sep 16 2003
    copyright            : (C) 2003 by Klemens Meyer
    email                : klesmen@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KALLELEMENTSDLG_H
#define KALLELEMENTSDLG_H

#include <qwidget.h>
#include <kdialogbase.h>
#include <klistview.h>
#include <kpushbutton.h>
#include <klineedit.h>
#include <qpopupmenu.h>
#include "memaid/memaid.h"


/**Extension of the KListViewItem to carry the original element from the NN code. */
class KElementItem	: public KListViewItem
{
public:

	KElementItem(struct elem* elmt, KListView* parent);

public:

	/** Sets wether the item has changed */
	bool Changed();

	/** Updates the changes made in the Dialog back to the element */
	void UpdateElement();
	/** Updated the changes made elsewhere in the element */
	void UpdateItem();

	/** Deliveres the element */
	struct elem* getElement()	{return element;}

protected:

	/** The source element wich is shown with this item */
	struct elem*	element;
	/** Indicates wether the shown text difers from the item */
	bool			changed;
};






/**A dialog for displaying the hole list of the Questions and answers (with inplace editing).
  *@author Klemens Meyer
  */

class KAllElementsDlg	: public KDialogBase
{
   Q_OBJECT
public:

	KAllElementsDlg(QWidget *parent=0, const char *name=0);
	~KAllElementsDlg();

protected:

	/** The view of the elements */
	KListView*	lview;

	/** Context-Menu for different things */
	QPopupMenu*	popup;

	/** Indicator wether an element has changed */
	bool changed;
	/** Item that is set when the popup-Menu is shown */
	KElementItem*	item;

    /** Edit element for finding elements */
    KLineEdit*      ftext;

	/** Fonts for the big-edit dialog */
	QFont	afont;
	QFont	qfont;

public slots: // Public slots
  /** The Item has changed */
  void slotEdited(QListViewItem* item, const QString&, int);
  /** Deletes the selected elements */
  void slotDelete();
  /** Shows the context-menu */
  void slotPopUp(KListView* lview, QListViewItem* item, const QPoint& p);
  /** Edits the element in the Edit-Elements Dialog. */
  void slotBigEdit();
  /** Takes the Element and puts a reversed answer-question element to the learning list. */
  void slotViceVersa();
  /** Sets the fonts for the Big-Edit dialog. */
  void setFonts(const QFont& q, const QFont& a);
  /** Finds an element in the list. */
  void slotFind();
  /** called when in the edit box return is pressed. */
  void slotFTextReturn(const QString&);
protected: // Protected methods
  /** Updates the Elements in the Listview back to the NN list. */
  void UpdateElements();
protected slots: // Protected slots
  /** No descriptions */
  virtual void slotCancel();
protected slots: // Protected slots
  /** No descriptions */
  virtual void slotApply();
protected slots: // Protected slots
  /** No descriptions */
  virtual void slotOk();
};

#endif
