/***************************************************************************
                          kflipcard.cpp  -  description
                             -------------------
    begin                : Sam Aug 16 2003
    copyright            : (C) 2003 by Klemens Meyer
    email                : klesmen@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kflipcard.h"
#include <qpainter.h>
#include <qrect.h>
#include <qpixmap.h>
#include <klocale.h>
#include <qsimplerichtext.h>
#include <qstylesheet.h>


KFlipCard::KFlipCard(QWidget *parent, const char *name )
	: QWidget(parent,name)
	, question(i18n("Question"))
	, answer(i18n("Answer"))
	, displayQuestion(true)
{
	question.sp_frame	= QColor("blue");
	answer.sp_frame		= QColor("green");

	setMinimumSize(200,80);
}

KFlipCard::~KFlipCard()
{
}

/** Zeichnen des Widgets */
void KFlipCard::paintEvent(QPaintEvent*)
{
	QRect		range(0, 0, width(), height());
	QPixmap		behind(range.size());
	QPainter	draw(&behind);

	drawCard(draw, range, displayQuestion ? question : answer);
	bitBlt(this, QPoint(0, 0) , &behind, range, Qt::CopyROP);
}

/** Prepares the string with tags for the display */
QString KFlipCard::prepareText(QString text)
{
	if(!QStyleSheet::mightBeRichText(text))
		text.replace(QChar('\n'), "<br>");

	return QString("<center>") + text + QString("</center>");
}

/** Display the card. */
void KFlipCard::drawCard(QPainter& painter, const QRect& range, const SideProps& attr)
{
	painter.fillRect(range, QBrush(attr.sp_card));
	painter.setPen(attr.sp_frame);
	painter.drawRect(range);

	QColorGroup		cg;

	cg.setColor(QColorGroup::Button, attr.sp_color);
	cg.setColor(QColorGroup::Text, attr.sp_color);
	cg.setColor(QColorGroup::Background, attr.sp_card);

	QSimpleRichText	rt(prepareText(attr.sp_text), attr.sp_font);

	rt.setWidth(&painter, range.width());

	int	x	= 0; // range.left() + (range.width() - rt.widthUsed()) / 2;
	int	y	= range.top() + (range.height() - rt.height()) / 2;

	rt.draw(&painter, x, y, range, cg);
/***
	painter.fillRect(range, QBrush(attr.sp_card));
	painter.setPen(attr.sp_frame);
	painter.drawRect(range);
	painter.setFont(attr.sp_font);
	painter.setPen(attr.sp_color);
	painter.drawText(range, AlignVCenter | AlignHCenter | WordBreak, attr.sp_text);
***/
}


/** Flips the displayed text */
void KFlipCard::flip()
{
	QPainter	painter(this);
	QRect		range(0,0, width(), height());

	if(displayQuestion)
		Squeeze(painter, range, question, answer);
	else
		Squeeze(painter, range, answer, question);

	displayQuestion	= !displayQuestion;
}

/** Squeezing the Card */
void KFlipCard::Squeeze(QPainter& dest, const QRect& range, const SideProps& front, const SideProps& back)
{
	QPixmap		behind(range.size());
	QPainter	draw(&behind);

	int			hx	= range.height() / 2;
	int			inc	= hx / 50 + 1;
	for(int i = 0 ; i < hx ; i += inc)
	{
		draw.fillRect(range, backgroundBrush());
		QRect	srange(range.left(), range.top() + i , range.width(), range.height() - 2 * i);
		drawCard(draw, srange, front);
		bitBlt(dest.device(), QPoint(0, 0) , &behind, range, Qt::CopyROP);
//		dest.drawPixmap(range.topLeft(), behind);
	}

	for(int i = hx ; i >= 0 ; i -= inc)
	{
		draw.fillRect(range, backgroundBrush());
		QRect	srange(range.left(), range.top() + i , range.width(), range.height() - 2 * i);
		drawCard(draw, srange, back);
		bitBlt(dest.device(), QPoint(0, 0) , &behind, range, Qt::CopyROP);
//		dest.drawPixmap(range.topLeft(), behind);
	}
}

/** Changes the Card with question and answer */
void KFlipCard::newCard(const QString& q, const QString& a, const QString& context)
{
	QRect		range(0, 0, width(), height());
	QPixmap		behind(range.size());
	QPainter	draw(&behind);
	SideProps&	front	= displayQuestion ? question : answer;

	int			hx	= range.height();
	int			inc	= hx / 50 + 1;
	for(int i = 0 ; i <hx ; i += inc)
	{
		draw.fillRect(range, backgroundBrush());
		QRect	srange(range.left(), range.top() + i , range.width(), range.height() + i);
		drawCard(draw, srange, front);
		bitBlt(this, QPoint(0, 0) , &behind, range, Qt::CopyROP);
	}

	displayQuestion		= true;
	question			= q;
	answer				= a;
	question.sp_context	= context;
	answer.sp_context	= context;
	for(int i = hx ; i >= 0 ; i -= inc)
	{
		draw.fillRect(range, backgroundBrush());
		QRect	srange(range.left(), range.top() + i , range.width(), range.height() + i);
		drawCard(draw, srange, question);
		bitBlt(this, QPoint(0, 0) , &behind, range, Qt::CopyROP);
	}
	repaint();
}

/** restores all the view-specific configuration */
void KFlipCard::readOptions(KConfig& config)
{
	config.setGroup("Card Options (Question Side)");
	question.readOptions(config);
	config.setGroup("Card Options (Answer Side)");
	answer.readOptions(config);
}

/** Writes all the view-specific configuration */
void KFlipCard::saveOptions(KConfig& config)
{
	config.setGroup("Card Options (Question Side)");
	question.saveOptions(config);
	config.setGroup("Card Options (Answer Side)");
	answer.saveOptions(config);
}
