/***************************************************************************
                          kflipcard.h  -  description
                             -------------------
    begin                : Sam Aug 16 2003
    copyright            : (C) 2003 by Klemens Meyer
    email                : klesmen@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KFLIPCARD_H
#define KFLIPCARD_H

#include <qwidget.h>
#include <kconfig.h>

/**Class for displaying a flipping filing-Card.
  *@author Klemens Meyer
  */

class KFlipCard : public QWidget
{
   Q_OBJECT

public:
	/** Collection-Class for the Properties of one card side */
	class SideProps
	{
	public:
		QString	sp_text;
		QString	sp_context;
		QFont	sp_font;
		QColor	sp_color;
		QColor	sp_card;
		QColor	sp_frame;

		/** Copy-Constructor */
		SideProps(const SideProps& right)
			: sp_text(right.sp_text)
			, sp_context(right.sp_context)
			, sp_font(right.sp_font)
			, sp_color(right.sp_color)
			, sp_card(right.sp_card)
			, sp_frame(right.sp_frame)
		{
		}
		/** Initializing */
		SideProps(const QString& text = "Text", const QString& context = QString::null, const QFont& font = QFont(), const QColor& color = QColor("black"), const QColor& card = QColor("white"), const QColor& frame = QColor("black"))
			: sp_text(text)
			, sp_context(context)
			, sp_font(font)
			, sp_color(color)
			, sp_card(card)
			, sp_frame(frame)
		{
		}

		/** Copying */
		SideProps& operator=(const SideProps& right)
		{
//			sp_text		= right.sp_text;
//			sp_context	= right.sp_context;
			sp_font		= right.sp_font;
			sp_color	= right.sp_color;
			sp_card		= right.sp_card;
			sp_frame	= right.sp_frame;
			return *this;
		}

		/** Setting the text */
		SideProps& operator=(const QString& right)
		{
			sp_text	= right;
			return *this;
		}
		/** Writing contents to Configuration */
		void saveOptions(KConfig& config)
		{
			config.writeEntry("Font", sp_font);
			config.writeEntry("Color", sp_color);
			config.writeEntry("Card", sp_card);
			config.writeEntry("Frame", sp_frame);
		}
		/** Reading contents to Configuration */
		void readOptions(KConfig& config)
		{
			sp_font		= config.readFontEntry("Font", &sp_font);
			sp_color	= config.readColorEntry("Color", &sp_color);
			sp_card		= config.readColorEntry("Card", &sp_card);
			sp_frame	= config.readColorEntry("Frame", &sp_frame);
		}
	};

public:

	KFlipCard(QWidget *parent=0, const char *name=0);
	~KFlipCard();

public:

	/** Properties of the Question */
	SideProps& Question()		{return question;}
	/** Properties of the Answer */
	SideProps& Answer()			{return answer;}
	/** Properties of the currently shown side */
	SideProps& Top()			{return displayQuestion ? question : answer;}
	/** Shows wether the question is displayed */
	bool isQuestion()			{return displayQuestion;}
	/** Sets wich card-Side will be displayed */
	void showQuestion(bool q)	{displayQuestion = q; repaint();}

	/** Changes the Card with question and answer */
	void newCard(const QString& question, const QString& answer, const QString& context);

	/** restores all the view-specific configuration */
	void readOptions(KConfig& config);
	/** Writes all the view-specific configuration */
	void saveOptions(KConfig& config);

protected: // Protected methods

  /** Zeichnen des Widgets */
  virtual void paintEvent(QPaintEvent*);
  /** Prepares the string with tags for the display */
  QString prepareText(QString text);
  /** Display the card. */
  void drawCard(QPainter& painter, const QRect& range, const SideProps& attr);

  /** Squeezing the Card */
  void Squeeze(QPainter& dest, const QRect& range, const SideProps& front, const SideProps& back);

protected: // Protected attributes

  /** Attributes of the question-side */
  SideProps	question;
  /** Attributes of the answer-side */
  SideProps answer;
  /** true when the question-side is displayed */
  bool displayQuestion;

public slots: // Public slots

  /** Flips the displayed text */
  void flip();

};



////////////////////////////////////////////////////////////////////////////
//   Inlines ab hier



#endif
