/***************************************************************************
                          kmemaid.cpp  -  description
                             -------------------
    begin                : Don Aug 21 12:20:37 CEST 2003
    copyright            : (C) 2003 by Klemens Meyer
    email                : klesmen@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kdirselectdialog.h>
#include <kmessagebox.h>
#include <kcursor.h>

// application specific includes
#include "kmemaid.h"
#include "kmemaidview.h"
#include "kmemaiddoc.h"
#include "kpreferences.h"
#include "knewdirdialog.h"
#include <unistd.h>
#include <pwd.h>
#include <sys/types.h>
#include <dirent.h>


#define ID_STATUS_MSG	1
#define ID_REPETITIONS	2
#define ID_ALL_ELEMENTS	3

KMemAidApp::KMemAidApp(QWidget* , const char* name):KMainWindow(0, name)
{
	config=kapp->config();

	///////////////////////////////////////////////////////////////////
	// call inits to invoke all other construction parts
	initStatusBar();
	initActions();
	initDocument();
	initView();
		
	readOptions();

}

KMemAidApp::~KMemAidApp()
{

}

void KMemAidApp::initActions()
{
  fileQuit = KStdAction::quit(this, SLOT(close()), actionCollection());
  fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
  fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
  fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
  fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
  viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());
  startLearn = new KAction(i18n("&Learn"), 0, this, SLOT(startLearning()), actionCollection(), "startLearning");
  stopLearn = new KAction(i18n("&Stop"), 0, this, SLOT(stopLearning()), actionCollection(), "stopLearning");
  addElement = new KAction(i18n("&Add elements"), 0, this, SLOT(addElements()), actionCollection(), "addElement");
  editElement = new KAction(i18n("&Edit single element"), 0, this, SLOT(editElements()), actionCollection(), "editElement");
  viewPreferences = KStdAction::preferences(this, SLOT(slotPreferences()), actionCollection());
  showAll = new KAction(i18n("Show all elements"), 0, this, SLOT(showAllElements()), actionCollection(), "showAllElements");
  fullStatistic = new KAction(i18n("Statistics"), 0, this, SLOT(slotFullStatistic()), actionCollection(), "fullStatistic");
  editCurrent = new KAction(i18n("Edit current element"), 0, this, SLOT(slotEditCurrent()), actionCollection(), "editCurrent");
  

  fileQuit->setStatusText(i18n("Closes KMemAid"));
  fileOpen->setStatusText(i18n("Opens data-files in a directory"));
  fileOpenRecent->setStatusText(i18n("Opens recent used documents"));
  viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));
  startLearn->setStatusText(i18n("Memorizing"));
  stopLearn->setStatusText(i18n("Stop memorizing"));
  addElement->setStatusText(i18n("Adding new elements"));
  editElement->setStatusText(i18n("Correcting single elements"));
  viewPreferences->setStatusText(i18n("Edit the settings"));
  showAll->setStatusText(i18n("Shows all Elements"));
  fullStatistic->setStatusText(i18n("Shows Statistic"));
  editCurrent->setStatusText(i18n("Edit current element"));

  // use the absolute path to your kmemaidui.rc file for testing purpose in createGUI();
//  createGUI("/home/km/projekte/kmemaid/kmemaid/kmemaidui.rc");
  createGUI();

}


void KMemAidApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG, 7);
  statusBar()->insertItem(i18n("I"), ID_REPETITIONS, 1);
  statusBar()->insertItem(i18n("I"), ID_ALL_ELEMENTS, 1);
}

void KMemAidApp::initDocument()
{
	doc = new KMemAidDoc(this);

	connect(doc, SIGNAL(dataReady(bool)), this, SLOT(slotDataReady(bool)));
	connect(doc, SIGNAL(endLearning()), this, SLOT(stopLearning()));
	connect(doc, SIGNAL(statusText(const QString&)), this, SLOT(slotStatusMsg(const QString&)));
	connect(doc, SIGNAL(statusToday(int)), this, SLOT(slotRepetitions(int)));
	connect(doc, SIGNAL(statusAll(int)), this, SLOT(slotAllElements(int)));

	doc->newDocument();
}

void KMemAidApp::initView()
{ 
	////////////////////////////////////////////////////////////////////
	// create the main widget here that is managed by KTMainWindow's view-region and
	// connect the widget to your document to display document contents.

	view = new KMemAidView(this);
	doc->addView(view);
	setCentralWidget(view);
	setCaption(doc->URL().fileName(),false);

	connect(view, SIGNAL(statusText(const QString&)), this, SLOT(slotStatusMsg(const QString&)));
}

void KMemAidApp::openDocumentFile(const KURL& url)
{
	KURL	nurl	= url;

	if(nurl.isEmpty())
	{
		QStringList	recent	= fileOpenRecent->items();
		if(!recent.isEmpty())
			nurl	= recent.first();
	}

	slotStatusMsg(i18n("Opening file..."));
	doc->openDocument(nurl);
    setCaption(nurl.fileName(), false);

	if(!nurl.isEmpty())
	{
		fileOpenRecent->addURL(nurl);
	}
}


KMemAidDoc *KMemAidApp::getDocument() const
{
  return doc;
}

void KMemAidApp::saveOptions()
{	
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", viewToolBar->isChecked());
  config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
  fileOpenRecent->saveEntries(config,"Recent Files");

  view->saveOptions(*config);
  doc->saveOptions(*config);
}


void KMemAidApp::readOptions()
{
   config->setGroup("General Options");
 
   // bar status settings
   bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
   viewToolBar->setChecked(bViewToolbar);
   slotViewToolBar();
 
  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();


  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);
	
  // initialize the recent file list
  fileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }

  view->readOptions(*config);
  doc->readOptions(*config);
}

void KMemAidApp::saveProperties(KConfig *_cfg)
{
  if(doc->URL().fileName()!=i18n("Untitled") && !doc->isModified())
  {
    // saving to tempfile not necessary

  }
  else
  {
    KURL url=doc->URL();	
    _cfg->writeEntry("filename", url.url());
    _cfg->writeEntry("modified", doc->isModified());
    QString tempname = kapp->tempSaveName(url.url());
    QString tempurl= KURL::encode_string(tempname);
    KURL _url(tempurl);
  }
}


void KMemAidApp::readProperties(KConfig* _cfg)
{
  QString filename = _cfg->readEntry("filename", "");
  KURL url(filename);
  bool modified = _cfg->readBoolEntry("modified", false);
  if(modified)
  {
    bool canRecover;
    QString tempname = kapp->checkRecoverFile(filename, canRecover);
    KURL _url(tempname);
  	
    if(canRecover)
    {
      doc->openDocument(_url);
      doc->setModified();
      setCaption(_url.fileName(),true);
      QFile::remove(tempname);
    }
  }
  else
  {
    if(!filename.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(),false);
    }
  }
}		

bool KMemAidApp::queryClose()
{
  doc->closeDocument();
  return true;
}

bool KMemAidApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KMemAidApp::slotFileNew()
{
//	KMessageBox::sorry(this, i18n("Can not create a new data.bin file in this program version!"));
//	return;

	slotStatusMsg(i18n("Creating new document..."));

	QString	url	= doc->URL().path();
	if(url.isEmpty())
	{
		struct passwd*	pwd	= getpwuid(getuid());
		if(pwd->pw_dir)
		{
			url	= pwd->pw_dir;
		}
		else
		{
			char	buf[1024];
			getcwd(buf, 1024);
			url	= buf;
		}
	}

	KNewDirDialog	dlg(url, true, this, i18n("Choose New Data Directory"));

	if(dlg.exec() == QDialog::Accepted)
	{
		doc->openDocument(dlg.url());
		setCaption(dlg.url().fileName(), false);
		fileOpenRecent->addURL( dlg.url() );
		return;
	}
	slotStatusMsg(i18n(""));
}

void KMemAidApp::slotFileOpen()
{
	slotStatusMsg(i18n("Opening file..."));
	
	KURL	 url	= KDirSelectDialog::selectDirectory(doc->URL().path(), true, this, i18n("Choose Data Directory"));
	if(!url.isEmpty())
	{
		if(url.path().right(1) != "/")
			url	= url.path() + "/";

		doc->openDocument(url);
		setCaption(url.fileName(), false);
		fileOpenRecent->addURL( url );
	}
	else
		slotStatusMsg(i18n("Opening canceled"));
}

void KMemAidApp::slotFileOpenRecent(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));
	
  doc->openDocument(url);
  setCaption(url.fileName(), false);
}

void KMemAidApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
	
  close();

  slotStatusMsg(i18n("Ready."));
}

void KMemAidApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggling toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }		

  slotStatusMsg(i18n("Ready."));
}

void KMemAidApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}

void KMemAidApp::slotStatusMsg(const QString &text)
{
	statusBar()->clear();
	statusBar()->changeItem(text, ID_STATUS_MSG);
}

/** Number of repetitions for today */
void KMemAidApp::slotRepetitions(int num)
{
	QString	snum;
	snum.setNum(num);
	statusBar()->changeItem(i18n("Today: ") + snum, ID_REPETITIONS);
}

/** Number of overall elements. */
void KMemAidApp::slotAllElements(int num)
{
	QString	snum;
	snum.setNum(num);
	statusBar()->changeItem(i18n("All: ") + snum, ID_ALL_ELEMENTS);
}


/** Starts the memorizing */
void KMemAidApp::startLearning()
{
	startLearn->setEnabled(false);
	stopLearn->setEnabled(true);
	addElement->setEnabled(false);
	editElement->setEnabled(false);
    editCurrent->setEnabled(true);
	doc->startLearning();
}
/** Stops the current memorizing */
void KMemAidApp::stopLearning()
{
	doc->stopLearning();
	startLearn->setEnabled(true);
	stopLearn->setEnabled(false);
	addElement->setEnabled(true);
	editElement->setEnabled(true);
    editCurrent->setEnabled(false);
}

/** This slot toggles the learn-actions in order to the loaded data. */
void KMemAidApp::slotDataReady(bool ready)
{
	startLearn->setEnabled(ready);
	stopLearn->setEnabled(false);
	addElement->setEnabled(ready);
	editElement->setEnabled(ready);
    editCurrent->setEnabled(false);
}

/** Editing the settings */
void KMemAidApp::slotPreferences()
{
	view->preferences();
}

/** Adding new elements */
void KMemAidApp::addElements()
{
	doc->addElements();
}

/** Correct elements from the database */
void KMemAidApp::editElements()
{
	doc->editElements();
}

/** Showing all Elements in the learning database. */
void KMemAidApp::showAllElements()
{
	QCursor	old	= cursor();
	setCursor(KCursor::waitCursor());
	doc->showAllElements();
	setCursor(old);
}

/** Gets the full statistical  information. */
void KMemAidApp::slotFullStatistic()
{
	doc->makeStatus(i18n("Full Status"), true);
}
/** Save Action. It saves, yes -- but it is not necessary though it starts training, too. */
void KMemAidApp::slotFileSave()
{
    doc->trainNN(true);
    ma_store();
}

/** Editing the current element in the memorizing. */
void KMemAidApp::slotEditCurrent()
{
    doc->editCurrent();
}
