/***************************************************************************
                          kmemaid.h  -  description
                             -------------------
    begin                : Don Aug 21 12:20:37 CEST 2003
    copyright            : (C) 2003 by Klemens Meyer
    email                : klesmen@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMEMAID_H
#define KMEMAID_H
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt

// include files for KDE 
#include <kapp.h>
#include <kmainwindow.h>
#include <kaccel.h>
#include <kaction.h>
#include <kurl.h>

// forward declaration of the KMemAid classes
class KMemAidDoc;
class KMemAidView;

/**
  * The base class for KMemAid application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of KMemAidView creates your center view, which is connected
  * to the window's Doc object.
  * KMemAidApp reimplements the methods that KMainWindow provides for main window handling and supports
  * full session management as well as using KActions.
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
  * @version KDevelop version 1.2 code generation
  */
class KMemAidApp : public KMainWindow
{
  Q_OBJECT

  friend class KMemAidView;

  public:
    /** construtor of KMemAidApp, calls all init functions to create the application.
     */
    KMemAidApp(QWidget* parent=0, const char* name=0);
    ~KMemAidApp();
    /** opens a file specified by commandline option
     */
	void openDocumentFile(const KURL& url = "");

    /** returns a pointer to the current document connected to the KTMainWindow instance and is used by
     * the View class to access the document object's methods
     */	
    KMemAidDoc *getDocument() const; 	

  protected:
    /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
     * file
     */ 	
    void saveOptions();
    /** read general Options again and initialize all variables like the recent file list
     */
    void readOptions();
    /** initializes the KActions of the application */
    void initActions();
    /** sets up the statusbar for the main window by initialzing a statuslabel.
     */
    void initStatusBar();
    /** initializes the document object of the main window that is connected to the view in initView().
     * @see initView();
     */
    void initDocument();
    /** creates the centerwidget of the KTMainWindow instance and sets it as the view
     */
    void initView();
    /** queryClose is called by KTMainWindow on each closeEvent of a window. Against the
     * @see KTMainWindow#queryClose
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryClose();
    /** queryExit is called by KTMainWindow when the last window of the application is going to be closed during the closeEvent().
     * Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's	
     * properties.
     * @see KTMainWindow#queryExit
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryExit();
    /** saves the window properties for each open window during session end to the session config file, including saving the currently
     * opened file by a temporary filename provided by KApplication.
     * @see KTMainWindow#saveProperties
     */
    virtual void saveProperties(KConfig *_cfg);
    /** reads the session config file and restores the application's state including the last opened files and documents by reading the
     * temporary files saved by saveProperties()
     * @see KTMainWindow#readProperties
     */
    virtual void readProperties(KConfig *_cfg);

  public slots:
    /** clears the document in the actual view to reuse it as the new document */
    void slotFileNew();
    /** open a file and load it into the document*/
    void slotFileOpen();
    /** opens a file from the recent files menu */
    void slotFileOpenRecent(const KURL& url);

    /** asks for saving if the file is modified, then closes the actual file and window*/
    void slotFileClose();
    /** toggles the toolbar
     */
    void slotViewToolBar();
    /** toggles the statusbar
     */
    void slotViewStatusBar();
    /** changes the statusbar contents for the standard label permanently, used to indicate current actions.
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusMsg(const QString &text);
//    void slotStatusMsg(const QString &text, int reps, int all);
  /** Starts the memorizing */
  void startLearning();
  /** Stops the current memorizing */
  void stopLearning();
  /** This slot toggles the learn-actions in order to the loaded data. */
  void slotDataReady(bool ready);
  /** Editing the settings */
  void slotPreferences();
  /** Adding new elements */
  void addElements();
  /** Correct elements from the database */
  void editElements();
  /** Showing all Elements in the learning database. */
  void showAllElements();
  /** Number of overall elements. */
  void slotAllElements(int num);
  /** Number of repetitions for today */
  void slotRepetitions(int num);
  /** Gets the full statistical  information. */
  void slotFullStatistic();
  /** Save Action. It saves, yes -- but it is not necessary though it starts training, too. */
  void slotFileSave();
  /** Editing the current element in the memorizing. */
  void slotEditCurrent();

  private:
    /** the configuration object of the application */
    KConfig *config;
    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
    KMemAidView *view;
    /** doc represents your actual document and is created only once. It keeps
     * information such as filename and does the serialization of your files.
     */
    KMemAidDoc *doc;

    // KAction pointers to enable/disable actions
    KAction* fileNew;
    KAction* fileOpen;
    KAction* fileQuit;
    KAction* fileSave;
    KRecentFilesAction* fileOpenRecent;
    KToggleAction* viewToolBar;
    KToggleAction* viewStatusBar;
    KAction* startLearn;
    KAction* stopLearn;
    KAction* addElement;
    KAction* editElement;
    KAction* viewPreferences;
    KAction* showAll;
    KAction* fullStatistic;
    KAction* editCurrent;
};
 
#endif // KMEMAID_H
