/***************************************************************************
                          kmemaiddoc.cpp  -  description
                             -------------------
    begin                : Don Aug 21 12:20:37 CEST 2003
    copyright            : (C) 2003 by Klemens Meyer
    email                : klesmen@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qwidget.h>
#include <qdatetime.h> 

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <kmessagebox.h>
#include <kprogress.h>

// application specific includes
#include "kmemaiddoc.h"
#include "kmemaid.h"
#include "kmemaidview.h"
#include "kqadialog.h"
#include <sys/stat.h>
#include "kallelementsdlg.h"
#include "memaid/memaid.h"


QList<KMemAidView> *KMemAidDoc::pViewList = 0L;

KMemAidDoc::KMemAidDoc(QWidget *parent, const char *name)
	: QObject(parent, name)
	, is_ma_init(false)
	, need_training(false)
	, train_time(1000)
	, state(LS_LAZY)
{
	if(!pViewList)
	{
		pViewList = new QList<KMemAidView>();
	}

	pViewList->setAutoDelete(true);
}

KMemAidDoc::~KMemAidDoc()
{
	deleteContents();
}

void KMemAidDoc::addView(KMemAidView *view)
{
  pViewList->append(view);
}

void KMemAidDoc::removeView(KMemAidView *view)
{
  pViewList->remove(view);
}
void KMemAidDoc::setURL(const KURL &url)
{
  doc_url=url;
}

const KURL& KMemAidDoc::URL() const
{
  return doc_url;
}

void KMemAidDoc::slotUpdateAllViews(KMemAidView *sender)
{
  KMemAidView *w;
  if(pViewList)
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
      if(w!=sender)
        w->repaint();
    }
  }

}

void KMemAidDoc::closeDocument()
{
  deleteContents();
}

bool KMemAidDoc::newDocument()
{
	deleteContents();

	return openDocument(QString::null);
}

bool KMemAidDoc::openDocument(const KURL& url, const char */*format =0*/)
{
	deleteContents();

	struct stat	stbuf;
	if(!url.isEmpty() && stat(url.path().ascii(), &stbuf) != 0)
	{
		QString	errm	= i18n("Data-Path\n") + url.path() + i18n("\ndoes not exist!");
		KMessageBox::error((QWidget*)parent(), errm, i18n("Opening Data Error"));
		doc_url.setFileName(i18n("Untitled"));
		makeStatus(i18n("No memory base chosen"), false);
		((QWidget*)parent())->setCaption("");
		doc_url.setFileName("");
	}
	else if(!url.isEmpty())
	{
		int	erg	= ma_init(url.path());
		if(erg & DIR_NOT_ACCESSIBLE)
		{
			KMessageBox::error((QWidget*)parent(), i18n("Data directory not accessible."), i18n("Opening Data Error"));
		}
		else
		{
			is_ma_init		= true;
			need_training	= false;
			doc_url			= url;
			emit dataReady(true);
		}
		makeStatus(i18n("Ready"), false);
	}
	else
	{
		doc_url.setFileName(i18n("Untitled"));
		makeStatus(i18n("No memory base chosen"), false);
	}

	modified=false;
	return true;
}

void KMemAidDoc::deleteContents()
{
	if(is_ma_init)
	{
		trainNN();
		ma_deinit();
	}
	is_ma_init	= false;

	emit dataReady(false);
}

/** Start to remember */
void KMemAidDoc::startLearning()
{
	static char	newq[MAX_Q_LEN];
	static char	newa[MAX_A_LEN];

	switch(state)
	{
		case LS_LAZY:
			state	= LS_REGULAR;
		case LS_REGULAR:
		case LS_FORCED:
		case LS_NEW:
//			element	= ma_el_to_repeat(state == LS_FORCED);
            element	= ma_el_to_repeat();
			if(element)
			{
				q	= QString::fromLocal8Bit(element->q);
				a	= QString::fromLocal8Bit(element->a);
				pViewList->first()->askQuestion(q, a, doc_url.path());
			}
			else if(state != LS_FORCED && ma_get_new_el(state == LS_NEW, newq, newa) == 0)
			{
				state	= LS_NEW;
				q		= QString::fromLocal8Bit(newq);
				a		= QString::fromLocal8Bit(newa);
				pViewList->first()->askQuestion(q, a, doc_url.path());
			}
			else
			{
//				int	nmat	= KMessageBox::questionYesNo((QWidget*)parent(), i18n("Nothing more to repeat. Would you like to learn in advance?"));
//				if(nmat == KMessageBox::Yes)
//				{
//					state	= LS_FORCED;
//					startLearning();
//				}
//				else
				{
					emit endLearning();
					makeStatus(i18n("Nothing more to learn"), false);
					state	= LS_LAZY;
				}
			}
			break;
		default:
			KMessageBox::error((QWidget*)parent(), i18n("This position is impossible to reach!"));
	 }
}
/** Stops the current memorizing */
void KMemAidDoc::stopLearning()
{
	state	= LS_LAZY;
	pViewList->first()->stopQestion();
}

/** Give a hint */
void KMemAidDoc::makeHint()
{
}

/** Setting a new grade for the current memorizing */
void KMemAidDoc::newGrade(int grade)
{
	if(state == LS_NEW)
		ma_make_new_el(grade, q.local8Bit(), a.local8Bit());
	else
		ma_move_el(element, grade);

	need_training	= true;
	startLearning();
}

/** Adds a new Question to the database */
void KMemAidDoc::newQuestion(const QString& question, const QString& answer, int grade)
{
   	ma_make_new_el(grade, question.local8Bit(), answer.local8Bit());
}

/** Adding new elements to the memorizing prozess */
void KMemAidDoc::addElements()
{
	KQADialog	dlg(true, i18n("New Elements"), (QWidget*)parent(), "QuestionAnswerDlg");

	dlg.setFonts(pViewList->first()->getQFont(), pViewList->first()->getAFont());

	connect(&dlg, SIGNAL(newQuestion(const QString&, const QString&, int)), this, SLOT(newQuestion(const QString&, const QString&, int)));
	dlg.exec();
	disconnect(&dlg, SIGNAL(newQuestion(const QString&, const QString&, int)), this, SLOT(newQuestion(const QString&, const QString&, int)));

	ma_store();
	
	makeStatus(i18n("Ready"), false);
}

/** Correct elements from the database */
void KMemAidDoc::editElements()
{
	KQADialog	dlg(false, i18n("Edit Elements"), (QWidget*)parent(), "QuestionAnswerDlg");

	dlg.setFonts(pViewList->first()->getQFont(), pViewList->first()->getAFont());

	dlg.exec();

	ma_store();

	makeStatus(i18n("Element added"), false);
}

/** Shows a dialog with all the elements in the learning database. */
void KMemAidDoc::showAllElements()
{
	KAllElementsDlg	dlg((QWidget*)parent(), "AllElementsDlg");

	dlg.setFonts(pViewList->first()->getQFont(), pViewList->first()->getAFont());

	dlg.exec();

	ma_store();

	makeStatus(i18n("Ready"), false);
}

/** Trains the net for sec seconds. */
void KMemAidDoc::trainNN(bool force)
{
	if(!need_training && !force)
		return;

	QTime	time;

	KProgressDialog*	prgrs	= new KProgressDialog((QWidget*)parent(), "trainprgrs", i18n("Training"), i18n("Training of the neural network in progress"));
	prgrs->setAutoClose(false);
	prgrs->setAllowCancel(false);
	prgrs->progressBar()->setTotalSteps(train_time);

	prgrs->show();
	KApplication::kApplication()->processEvents();

	time.start();
	while(time.elapsed() <= train_time)
	{
		prgrs->progressBar()->setValue(time.elapsed());
		KApplication::kApplication()->processEvents();
		ma_train_ann(6);
	}
	delete prgrs;
	need_training	= false;
}

/** Computes the status and displays it according to the parameter full.
If full == false then only the status bar in the main window is updated.
With full == true a dialog is shown with all collected information. */
void KMemAidDoc::makeStatus(const QString& txt, bool full)
{
	if(!is_ma_init)
	{
		emit statusText(i18n("No memory base chosen"));
		emit statusAll(0);
		emit statusToday(0);
	}
	else
	{
		struct ma_stats_st	stats;
		ma_stats(&stats);

		emit statusText(txt);
		emit statusAll(stats.total_items);
		emit statusToday(stats.rpts_for_today);

		if(full)
		{
			QString s;

			s = i18n( "Total number of elements: %1 \n" ).arg( stats.total_items );
			s.append( i18n("Number of repetitions for today: %1 \n ").arg( stats.rpts_for_today ) );
			s.append ( i18n("Number of days since first run: %1 \n").arg( stats.days ));
			s.append ( i18n("Max current interval: %1 \n").arg( stats.max_ivl ) );
			s.append ( i18n("Min current interval: %1 \n").arg(  stats.min_ivl ));
			s.append ( i18n("Mean current interval: %1 \n").arg( stats.mean_ivl ) );
			s.append ( i18n("Max last interval by ann - before last review of every item: %1 \n").arg( stats.max_l_ivl_ann ));		/*  */
			s.append ( i18n("Min last interval by ann - before last review of every item: %1 \n").arg( stats.min_l_ivl_ann ));
			s.append ( i18n("Mean last interval by ann - before last review of every item: %1 \n").arg( stats.mean_l_ivl_ann ));
			s.append ( i18n("Max real last interval (we don't know current real interval yet): %1 \n").arg( stats.max_l_ivl_real ));		/*  */
			s.append ( i18n("Min real last interval (we don't know current real interval yet): %1 \n").arg( stats.min_l_ivl_real ));
			s.append ( i18n("Mean real last interval (we don't know current real interval yet): %1 \n").arg( stats.mean_l_ivl_real ) );

			s.append ( i18n("Mean grade: %1 \n").arg( stats.mean_grade ));	/* (sum_grades/total_items) */

			s.append ( i18n("Mean reviews: %1 \n").arg( stats.mean_reviews ));
			s.append ( i18n("Sum reviews: %1 \n").arg( stats.sum_reviews ) );	/* how many total reviews,
																				 *  not counting final drill and repetition ahead of time
																				 * (e.g. final drill)
																				 */

			s.append ( i18n("Max reviews: %1 \n").arg( stats.max_reviews ));		/* max number of reviews of an item so far */

			s.append ( i18n("ANN error: %1 \n").arg( stats.ann_error ) );	/* ANN error, rather only for debug,
																			 * or just to sure that everything is ok
																			 */
			s.append ( i18n("ANN cases: %1 \n").arg( stats.ann_cases ));	/* total number of cases to train ANN */
			s.append ( i18n("ANN user cases: %1 \n").arg( stats.user_ann_cases ) );	/* number of cases gathered from the user */


			s.append ( i18n("Retention: %1 \n").arg( stats.retention ));	/* aproximate retention, proportion:
																			 * (items_you_remember/total_items)
																			 * (items_you_remember = items that had grade 3 or higher
																			 * at last repetition
																			 */

			s.append ( i18n("Items per day: %1 \n").arg( stats.item_per_day ));		/* New items per day */
			s.append ( i18n("First run: %1 \n").arg( stats.first_run ) );		/* exact time/date of first run */

			KMessageBox::information((QWidget*)parent(),  s );
		}
	}
}
/** Sets the training-time for the NN in millisecs */
void KMemAidDoc::setTrainTime(int msecs)
{
	train_time	= msecs;
}

/** Restores all the document specific configuration */
void KMemAidDoc::readOptions(KConfig& config)
{
	config.setGroup("Document Options");
	train_time	= config.readNumEntry("Train Time", train_time);
}

/** Writes all the document specific configuration */
void KMemAidDoc::saveOptions(KConfig& config)
{
	config.setGroup("Document Options");
	config.writeEntry("Train Time", train_time);
}
/** Editing the current element in the memorizing. */
void KMemAidDoc::editCurrent()
{
    KQADialog   dlg(false, i18n("Editing Current Element"), (QWidget*)parent(), "editcurrent", element);

    dlg.setFonts(pViewList->first()->getQFont(), pViewList->first()->getAFont());

    dlg.exec();

    ma_store();

	q	= QString::fromLocal8Bit(element->q);
	a	= QString::fromLocal8Bit(element->a);

    pViewList->first()->askQuestion(q, a, doc_url.path());
}
