/***************************************************************************
                          kmemaiddoc.h  -  description
                             -------------------
    begin                : Don Aug 21 12:20:37 CEST 2003
    copyright            : (C) 2003 by Klemens Meyer
    email                : klesmen@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMEMAIDDOC_H
#define KMEMAIDDOC_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qlist.h>

// include files for KDE
#include <kurl.h>
#include <kconfig.h>

#include "memaid/memaid.h"

// forward declaration of the KMemAid classes
class KMemAidView;

/**	KMemAidDoc provides a document object for a document-view model.
  *
  * The KMemAidDoc class provides a document object that can be used in conjunction with the classes KMemAidApp and KMemAidView
  * to create a document-view model for standard KDE applications based on KApplication and KMainWindow. Thereby, the document object
  * is created by the KMemAidApp instance and contains the document structure with the according methods for manipulation of the document
  * data by KMemAidView objects. Also, KMemAidDoc contains the methods for serialization of the document data from and to files.
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team. 	
  * @version KDevelop version 1.2 code generation
  */
class KMemAidDoc : public QObject
{
  Q_OBJECT
  public:

  	/** Status values during the learning process */
	enum LearnState	{ LS_LAZY		/**< No learning in progress yet. */
					, LS_REGULAR	/**< Regular repetitions are made. */
					, LS_NEW		/**< Elements from the text file are processed. */
					, LS_FORCED		/**< Elements not yet sceduled are learned. */
					};
  
  public:
    /** Constructor for the fileclass of the application */
    KMemAidDoc(QWidget *parent, const char *name=0);
    /** Destructor for the fileclass of the application */
    ~KMemAidDoc();

    /** adds a view to the document which represents the document contents. Usually this is your main view. */
    void addView(KMemAidView *view);
    /** removes a view from the list of currently connected views */
    void removeView(KMemAidView *view);
    /** sets the modified flag for the document after a modifying action on the view connected to the document.*/
    void setModified(bool _m=true){ modified=_m; };
    /** returns if the document is modified or not. Use this to determine if your document needs saving by the user on closing.*/
    bool isModified(){ return modified; };
    /** deletes the document's contents */
    void deleteContents();
    /** initializes the document generally */
    bool newDocument();
    /** closes the acutal document */
    void closeDocument();
    /** loads the document by filename and format and emits the updateViews() signal */
    bool openDocument(const KURL& url, const char *format=0);

    /** returns the KURL of the document */
    const KURL& URL() const;
    /** sets the URL of the document */
	  void setURL(const KURL& url);

	int trainTime()	{return train_time;}
	/** Writes all the document specific configuration */
	void saveOptions(KConfig& config);
	/** Restores all the document specific configuration */
	void readOptions(KConfig& config);

  public slots:
    /** calls repaint() on all views connected to the document object and is called by the view by which the document has been changed.
     * As this view normally repaints itself, it is excluded from the paintEvent.
     */
    void slotUpdateAllViews(KMemAidView *sender);
  /** Start to remember */
  void startLearning();
  /** Stops the current memorizing */
  void stopLearning();
  /** Give a hint */
  void makeHint();
  /** Setting a new grade for the current memorizing */
  void newGrade(int grade);
  /** Adds a new Question to the database */
  void newQuestion(const QString& question, const QString& answer, int grade);
  /** Adding new elements to the memorizing prozess */
  void addElements();
  /** Correct elements from the database */
  void editElements();
  /** Shows a dialog with all the elements in the learning database. */
  void showAllElements();
  /** Trains the net for sec seconds. */
  void trainNN(bool force = false);
  /** Computes the status and displays it according to the parameter full.
	If full == false then only the status bar in the main window is updated.
	With full == true a dialog is shown with all collected information. */
  void makeStatus(const QString& status, bool full);
  /** Sets the training-time for the NN in millisecs */
  void setTrainTime(int msecs);
  /** Editing the current element in the memorizing. */
  void editCurrent();
 	
signals: // Signals
  /** emits a signal wether the data is loaded or not. */
  void dataReady(bool ready);
  /** This signal is emitted when the memorizing has ended. */
  void endLearning();
  /** What to display in the statusbar. */
  void statusText(const QString& text);
  /** Emitting how many repetitions for today */
  void statusToday(int nums);
  /** Emiting how many Items in the learning list. */
  void statusAll(int nums);


  public:	
    /** the list of the views currently connected to the document */
    static QList<KMemAidView> *pViewList;	

  private:
    /** the modified flag of the current document */
    bool modified;
    KURL doc_url;
    bool is_ma_init;
protected: // Protected attributes
	/** Holds the current element to memorize */
	elem* element;
	/** Indicates wether the ANN has to trained */
	bool need_training;
	/** How long to train */
	int train_time;
	/** Current learning-state in progress */
	LearnState	state;
	/** Currently asked question */
	QString		q,a;
};

#endif // KMEMAIDDOC_H
