/***************************************************************************
                          kmemaidview.cpp  -  description
                             -------------------
    begin                : Don Aug 21 12:20:37 CEST 2003
    copyright            : (C) 2003 by Klemens Meyer
    email                : klesmen@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qprinter.h>
#include <qpainter.h>
#include <klocale.h>
#include <qradiobutton.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kaboutdata.h>

// application specific includes
#include "kmemaidview.h"
#include "kmemaiddoc.h"
#include "kmemaid.h"
#include "kpreferences.h"


KMemAidView::KMemAidView(QWidget *parent, const char *name)
	: QVBox(parent, name)
	, interval(3000)
{
//	setBackgroundMode(PaletteBase);

	flipcard	= new KFlipCard(this, "Question/Answer");
	QHBox*	hb	= new QHBox(this);
	flip		= new KPushButton(i18n("Flip"), hb, "flip");
	hint		= new KPushButton(i18n("Hint"), hb, "hint");
	grades		= new QHButtonGroup(i18n("Grade-Select"), this, "GradeSelect");
	timer		= new QTimer(this, "ActivateTimer");

	for(int i = 0 ; i < 6 ; ++i)
	{
		QString	title;
		title.sprintf("%d", i);

		KPushButton*	p	= new KPushButton(title, grades, title);
		p->setAccel(QKeySequence(title));
	}

	flip->setAccel(QKeySequence(Key_Enter));
	hint->setAccel(QKeySequence(Key_Plus));

	hb->setSpacing(20);
	hb->setStretchFactor(flip, 3);
	hb->setStretchFactor(hint, 2);
	hb->setFixedHeight(hb->sizeHint().height());
	grades->setFixedHeight(grades->sizeHint().height());
	
	connect(flip, SIGNAL(clicked()), flipcard, SLOT(flip()));
	connect(flip, SIGNAL(clicked()), this, SLOT(onFlip()));
	connect(hint, SIGNAL(clicked()), this, SLOT(makeHint()));

	connect(grades, SIGNAL(clicked(int)), this, SLOT(onNewGrade(int)));

	connect(timer, SIGNAL(timeout()), this, SLOT(activate()));

	flipcard->Question()	= welcomeMessage();
	flipcard->Answer()		= "";

	flip->setEnabled(false);
	hint->setEnabled(false);
	grades->setEnabled(false);
}

KMemAidView::~KMemAidView()
{
}

KMemAidDoc *KMemAidView::getDocument() const
{
  KMemAidApp *theApp=(KMemAidApp *) parentWidget();

  return theApp->getDocument();
}

/** Starting the learn-request for one fact. */
void KMemAidView::askQuestion(const QString& question, const QString& answer, const QString& context)
{
	flipcard->newCard(question, answer, context);

	if(interval)
	{
		getDocument()->makeStatus(i18n("What could that be?"), false);
		timer->start(interval, true);
	}
	else
	{
		activate();
	}
}

/** activating the flip-buttons after timer */
void KMemAidView::activate()
{
	flip->setEnabled(true);
	hint->setEnabled(true);

	show_hint	= 0;

	emit statusText(i18n("Flip the card to see the answer"));
}

/** When the card is flipped, then the grades must be chosen. */
void KMemAidView::onFlip()
{
	grades->setEnabled(true);
	hint->setEnabled(false);
	emit statusText(i18n("Pick a grade - how well you rememember the fact. (5 is the best)"));
}

/** Stopping the memorizing */
void KMemAidView::stopQestion()
{
	timer->stop();
	flipcard->newCard(welcomeMessage(), "", QString::null);
	flip->setEnabled(false);
	hint->setEnabled(false);
	grades->setEnabled(false);
	getDocument()->makeStatus(i18n("Memorizing stopped"), false);
}

/** get the new Grade */
void KMemAidView::onNewGrade(int grade)
{
	flip->setEnabled(false);
	hint->setEnabled(false);
	grades->setEnabled(false);
	getDocument()->newGrade(grade);
}
/** Writes all the view-specific configuration */
void KMemAidView::saveOptions(KConfig& config)
{
	flipcard->saveOptions(config);

	config.setGroup("Card Options");
	config.writeEntry("Flip Delay", interval);
}

/** restores all the view-specific configuration */
void KMemAidView::readOptions(KConfig& config)
{
	flipcard->readOptions(config);

	config.setGroup("Card Options");
	interval	= config.readNumEntry("Flip Delay", interval);
}
/** Changing the Settings */
void KMemAidView::preferences()
{
	prefs	= new KPreferences(i18n("Preferences"), this, "Preferences");

    connect(prefs, SIGNAL(applyClicked()), this, SLOT(prefApply()));
	
	prefs->Question()	= flipcard->Question();
	prefs->Answer()		= flipcard->Answer();
	prefs->Interval()	= interval;
	prefs->trainTime()	= getDocument()->trainTime();

	if(prefs->exec() == QDialog::Accepted)
	{
		flipcard->Question()	= prefs->Question();
		flipcard->Answer()		= prefs->Answer();
		interval				= prefs->Interval();
		getDocument()->setTrainTime(prefs->trainTime());

		flipcard->repaint();
	}
	delete prefs;
}
/** Applying the options from the options dialog */
void KMemAidView::prefApply()
{
	flipcard->Question()	= prefs->Question();
	flipcard->Answer()		= prefs->Answer();

	flipcard->repaint();
}
/** Displaying a cripled answer */
void KMemAidView::makeHint()
{
	static QString	whites(" \t\n\r");
	if(flipcard->isQuestion())
	{
		temp_answer	= flipcard->Answer().sp_text;
		show_hint += 10;
		if (show_hint > 100)
			show_hint = 100;

		QString tmp	= temp_answer;
		for (unsigned int i = 0; i < tmp.length(); i++)
		{
			if(whites.find(tmp[i]) < 0 && rand()%101 > show_hint)
				tmp[i] = '_';
		}
		flipcard->Answer()	= tmp;
		flip->setEnabled(false);
		hint->setText(i18n("Back"));
        hint->setAccel(QKeySequence(Key_Plus));
		flipcard->flip();
	}
	else
	{
		flipcard->flip();
		flipcard->Answer()	= temp_answer;
		hint->setText(i18n("Hint"));
        hint->setAccel(QKeySequence(Key_Plus));
		flip->setEnabled(true);
	}
}
/** Creates the welcome-message to display on the card when no questioning is performed. */
QString KMemAidView::welcomeMessage()
{
	const KAboutData*	about	= KApplication::kApplication()->aboutData();


	return QString("<table border=1>")
		+  QString("<tr>")
		+  QString("<td colspan=2 align=center>") + i18n("Welcome to ") + about->programName() + QString("</td>")
		+  QString("</tr>")
		+  QString("<tr>")
		+  QString("<td align=center>") + i18n("Version") + QString("</td>")
		+  QString("<td align=center>") + about->version() + QString("</td>")
		+  QString("</tr>")
		+  QString("</table>")
		;
}
