/***************************************************************************
                          kmemaidview.h  -  description
                             -------------------
    begin                : Don Aug 21 12:20:37 CEST 2003
    copyright            : (C) 2003 by Klemens Meyer
    email                : klesmen@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMEMAIDVIEW_H
#define KMEMAIDVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for Qt
#include <qwidget.h>
#include <qvbox.h>
#include "kflipcard.h"
#include <kpushbutton.h>
#include <qhbuttongroup.h>
#include "kpreferences.h"
#include <qtimer.h>


class KMemAidDoc;

/** The KMemAidView class provides the view widget for the KMemAidApp instance.	
 * The View instance inherits QWidget as a base class and represents the view object of a KTMainWindow. As KMemAidView is part of the
 * docuement-view model, it needs a reference to the document object connected with it by the KMemAidApp class to manipulate and display
 * the document structure provided by the KMemAidDoc class.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 0.4 code generation
 */
class KMemAidView : public QVBox
{
  Q_OBJECT
public:
    /** Constructor for the main view */
    KMemAidView(QWidget *parent = 0, const char *name=0);
    /** Destructor for the main view */
    ~KMemAidView();

    /** returns a pointer to the document connected to the view instance. Mind that this method requires a KMemAidApp instance as a parent
     * widget to get to the window document pointer by calling the KMemAidApp::getDocument() method.
     *
     * @see KMemAidApp#getDocument
     */
    KMemAidDoc *getDocument() const;
	/** restores all the view-specific configuration */
	void readOptions(KConfig& config);
	/** Writes all the view-specific configuration */
	void saveOptions(KConfig& config);

	/** Get the question font */
	const QFont& getQFont()	{return flipcard->Question().sp_font;}
	/** Get the answer font */
	const QFont& getAFont()	{return flipcard->Answer().sp_font;}

	
private:
	
protected: // Protected attributes
  /** Displaying the Question/Answer Card */
  KFlipCard*		flipcard;
  KPushButton*		flip;
  KPushButton*		hint;
  QHButtonGroup*	grades;
  /** Options changing dialog */
  KPreferences*		prefs;
  /** Grade of the hint display */
  int				show_hint;
  /** store the answer temporary when showing the hint */
  QString			temp_answer;
  /** Timer to let the user wait before flipping the card */
  QTimer*			timer;
  /** how long to wait before flipping */
  int				interval;

public slots: // Public slots
  /** Starting the learn-request for one fact. */
  void askQuestion(const QString& question, const QString& answer, const QString& context);
  /** When the card is flipped, then the grades must be chosen. */
  void onFlip();
  /** Stopping the memorizing */
  void stopQestion();
  /** get the new Grade */
  void onNewGrade(int grade);
  /** Changing the Settings */
  void preferences();
  /** Applying the options from the options dialog */
  void prefApply();
  /** Displaying a cripled answer */
  void makeHint();
  /** activating the flip-buttons after timer */
  void activate();
signals: // Signals
  /** Notifying all who want to know about the new grade of the single question. */
  void newGrade(int grade);
  /** Signal for the changing of the status-text */
  void statusText(const QString& text);
protected: // Protected methods
  /** Creates the welcome-message to display on the card when no questioning is performed. */
  QString welcomeMessage();
};

#endif // KMEMAIDVIEW_H
