/***************************************************************************
                          kpreferences.cpp  -  description
                             -------------------
    begin                : Son Aug 24 2003
    copyright            : (C) 2003 by Klemens Meyer
    email                : klesmen@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpreferences.h"
#include <qvbox.h>
#include <qhbox.h>
#include <kpushbutton.h>
#include <klocale.h>
#include <qlabel.h>
#include <kfontdialog.h>
#include <kcolordialog.h>
#include <qhgroupbox.h>


KPreferences::KPreferences(const QString& caption, QWidget *parent, const char *name )
	: KDialogBase(parent, name, true, caption, Ok|Apply|Cancel)
{
	QVBox*	page	= makeVBoxMainWidget();

	QHGroupBox*	cardgrp	= new QHGroupBox(i18n("Card Appearance"), page, "cardpage");
	QHBox*	cardpage	= new QHBox(cardgrp, "cardpage");
	QVBox*	cardside	= new QVBox(cardpage, "cardside");
	QVBox*	change		= new QVBox(cardpage, "change");
	flipcard			= new KFlipCard(cardside, "flipcard");
	KPushButton* flip	= new KPushButton(i18n("Flip"), cardside, "flip");
	KPushButton* chfont	= new KPushButton(i18n("Change Font..."), change, "change_font");
	KPushButton* chcol	= new KPushButton(i18n("Change Text Color..."), change, "change_text_color");
	KPushButton* chcard	= new KPushButton(i18n("Change Card Color..."), change, "change_card_color");
	KPushButton* chfrm	= new KPushButton(i18n("Change Frame Color..."), change, "change_frame_color");

	QHGroupBox* waitpage= new QHGroupBox(i18n("First Flip Delay"), page, "waitpage");
	waitslider			= new QSlider(0, 100, 10, 30, Horizontal, waitpage, "waitslider");
	waitdisp			= new QLCDNumber(4, waitpage, "waitlcd");

	QHGroupBox*	trainpage	= new QHGroupBox(i18n("NN Training Time"), page, "trainpage");
	trainslider				= new QSlider(0, 100, 10, 30, Horizontal, trainpage, "trainslider");
	traindisp				= new QLCDNumber(4, trainpage, "traindisp");

	cardpage->setSpacing(20);
	cardside->setSpacing(2);
	change->setSpacing(6);

	cardpage->setStretchFactor(cardside, 5);
	cardpage->setStretchFactor(change, 1);
	
	flipcard->Question()	= i18n("Question side");
	flipcard->Answer()		= i18n("Answer side");

	waitslider->setTickmarks(QSlider::Below);
	waitslider->setTickInterval(10);

	trainslider->setTickmarks(QSlider::Below);
	trainslider->setTickInterval(10);

	flip->setFixedHeight(flip->sizeHint().height());
	waitdisp->setFixedSize(waitdisp->sizeHint() * 1.5);
	traindisp->setFixedSize(waitdisp->sizeHint() * 1.5);

	setFixedSize(sizeHint());

	connect(flip, SIGNAL(clicked()), flipcard, SLOT(flip()));
	connect(chfont, SIGNAL(clicked()), this, SLOT(changeCardFont()));
	connect(chcol, SIGNAL(clicked()), this, SLOT(changeTextColor()));
	connect(chcard, SIGNAL(clicked()), this, SLOT(changeCardColor()));
	connect(chfrm, SIGNAL(clicked()), this, SLOT(changeFrameColor()));
	connect(waitslider, SIGNAL(valueChanged(int)), this, SLOT(waitSlot(int)));
	connect(trainslider, SIGNAL(valueChanged(int)), this, SLOT(trainSlot(int)));
}

KPreferences::~KPreferences()
{
}

/** setting of the original options */
void KPreferences::setOptions(KFlipCard& orig, int waitflip)
{
	if(flipcard)
	{
		flipcard->Question()	= orig.Question();
		flipcard->Answer()		= orig.Answer();
	}
	if(waitslider && waitdisp)
	{
		waitslider->setValue(waitflip / 100);
		waitdisp->display(float(waitflip) / 1000);
	}
}


/** Changing the font of the Card */
void KPreferences::changeCardFont()
{
	QFont	newFont(flipcard->Top().sp_font);
	if(KFontDialog::getFont(newFont, this) == KFontDialog::Accepted)
	{
		flipcard->Top().sp_font	= newFont;
		flipcard->repaint();
	}
}
/** No descriptions */
void KPreferences::changeTextColor()
{
	QColor	newColor(flipcard->Top().sp_color);
	if(KColorDialog::getColor(newColor, this) == KColorDialog::Accepted)
	{
		flipcard->Top().sp_color	= newColor;
		flipcard->repaint();
	}
}
/** Color of the card */
void KPreferences::changeCardColor()
{
	QColor	newColor(flipcard->Top().sp_card);
	if(KColorDialog::getColor(newColor, this) == KColorDialog::Accepted)
	{
		flipcard->Top().sp_card	= newColor;
		flipcard->repaint();
	}
}
/** Color of the Frame */
void KPreferences::changeFrameColor()
{
	QColor	newColor(flipcard->Top().sp_frame);
	if(KColorDialog::getColor(newColor, this) == KColorDialog::Accepted)
	{
		flipcard->Top().sp_frame	= newColor;
		flipcard->repaint();
	}
}

/** Change the Display value of the waitdisp */
void KPreferences::waitSlot(int val)
{
	waitdisp->display(float(val) / 10.0);
}

/** Setting the value for the training time. */
void KPreferences::trainSlot(int val)
{
	traindisp->display(float(val) / 10.0);
}
