/***************************************************************************
                          kpreferences.h  -  description
                             -------------------
    begin                : Son Aug 24 2003
    copyright            : (C) 2003 by Klemens Meyer
    email                : klesmen@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPREFERENCES_H
#define KPREFERENCES_H

#include <qwidget.h>
#include <kdialogbase.h>
#include "kflipcard.h"
#include <qslider.h>
#include <qlcdnumber.h>


/** Reference to deliver/change the wait interval */
class WaitReference
{
public:

	WaitReference(QSlider* s, QLCDNumber* d)
		: waitslider(s), waitdisp(d)
		{};
	WaitReference(const WaitReference& right)
		: waitslider(right.waitslider), waitdisp(right.waitdisp)
		{};

public:
	operator int()
	{
		return waitslider->value() * 100;
	}
	
	WaitReference& operator=(int msecs)
	{
		waitslider->setValue(msecs / 100);
		waitdisp->display(float(msecs) / 1000);
		return *this;
	}
protected:
	QSlider*	waitslider;
	QLCDNumber*	waitdisp;
};


/**Dialog for changing the settings of KMemAid
  *@author Klemens Meyer
  */

class KPreferences : public KDialogBase
{
   Q_OBJECT
public:


public:

	KPreferences(const QString& caption, QWidget *parent=0, const char *name=0);
	~KPreferences();

public:

	/** setting of the original options */
	void setOptions(KFlipCard& orig, int waitflip);

	/** get question side prefs */
	KFlipCard::SideProps& Question()	{return flipcard->Question();}
	/** get answer side prefs */
	KFlipCard::SideProps& Answer()		{return flipcard->Answer();}
	/** get waitinterval */
	WaitReference Interval()			{return WaitReference(waitslider, waitdisp);}
	/** get traininterval */
	WaitReference trainTime()			{return WaitReference(trainslider, traindisp);}
	
protected:

	/** Visible testobject for the changes */
	KFlipCard*	flipcard;

	QSlider*	waitslider;
	QLCDNumber*	waitdisp;

	QSlider*	trainslider;
	QLCDNumber*	traindisp;
	
public slots: // Public slots
  /** Changing the font of the Card */
  void changeCardFont();
  /** No descriptions */
  void changeTextColor();
  /** Color of the Frame */
  void changeFrameColor();
  /** Color of the card */
  void changeCardColor();
  /** Change the Display value of the waitdisp */
  void waitSlot(int val);
  /** Setting the value for the training time. */
  void trainSlot(int val);
};

#endif
