/***************************************************************************
                          kqadialog.cpp  -  description
                             -------------------
    begin                : Sam Aug 23 2003
    copyright            : (C) 2003 by Klemens Meyer
    email                : klesmen@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kqadialog.h"
#include <klocale.h>
#include <qvbox.h>
#include <kpushbutton.h>
#include <qlabel.h>
#include <kmessagebox.h>



KQADialog::KQADialog(bool adding, const QString& caption, QWidget *parent, const char *name, elem* el )
	: KDialogBase(parent, name, true, caption, Close)
	, viceversa(NULL)
	, element(el)
	, corr_close(!adding && el)
{
	QVBox*	page	= makeVBoxMainWidget();

    QHButtonGroup*  qbox    = new QHButtonGroup(i18n("Question"), page, "questiongroup");
    QHButtonGroup*  abox    = new QHButtonGroup(i18n("Answer"), page, "answergroup");

	question	= new KTextEdit("", QString::null, qbox, "question");
	answer		= new KTextEdit("", QString::null, abox, "answer");

	if(adding)
	{
		viceversa	= new QCheckBox(i18n("Add Vice Versa too"), page, "viceversa");
		grades		= new QHButtonGroup(i18n("Grade Select"), page, "GradeSelect");

		for(int i = 0 ; i < 6 ; ++i)
		{
			QString	title;
			title.sprintf("%d", i);

			KPushButton*	pb	= new KPushButton(title, grades, title);
			pb->setAccel(QKeySequence(title));
		}
		grades->setFixedHeight(grades->sizeHint().height());

		connect(grades, SIGNAL(clicked(int)), this, SLOT(newGrade(int)));

		statusBar	= new KStatusBar(page, "status");
		statusBar->insertItem(i18n("Type question and answer, then choose an initial Grade (5 is the best)."), 0, 1);
	}
	else
	{
		QHBox*	box	= new QHBox(page, "findbox");
		QString	title(element ? i18n("Correct") : i18n("Find"));
		fcbutton	= new KPushButton(title, box, "find_correct_button");
		box->setFixedHeight(box->sizeHint().height());

		connect(fcbutton, SIGNAL(clicked()), this, SLOT(findCorrect()));

		statusBar	= new KStatusBar(page, "status");
		statusBar->insertItem(i18n("Type question and/or answer, then click on find."), 0, 1);
	}

	question->setTabChangesFocus(true);
	answer->setTabChangesFocus(true);

	if(element)
	{
		question->setText(QString::fromLocal8Bit(element->q));
		answer->setText(QString::fromLocal8Bit(element->a));
	}

}

KQADialog::~KQADialog()
{
}

/** Select the new Grade */
void KQADialog::newGrade(int grade)
{
	ngrade	= grade;
	emit newQuestion(question->text(), answer->text(), grade);

	if(viceversa && viceversa->isChecked())
		emit newQuestion(answer->text(), question->text(), grade);

	question->setText("");
	answer->setText("");

	question->setFocus();
}

/** Finding and correcting of elements */
void KQADialog::findCorrect()
{
	QString	q	= question->text().local8Bit();
	QString	a	= answer->text().local8Bit();

	if(element)
	{
		element->q = (char*) realloc (element->q, (size_t)(q.length()+1));
		element->a = (char*) realloc (element->a, (size_t)(a.length()+1));
		strcpy (element->q, q);
		strcpy (element->a, a);

		element	= NULL;
		
		statusBar->changeItem(i18n("Type question and/or answer then click on find."), 0);
		fcbutton->setText(i18n("Find"));
		question->setText("");
		answer->setText("");

		question->setFocus();
		if(corr_close)
			close();
	}
	else
	{
		element	= ma_find_el(q, a);
		if(element)
		{
			statusBar->changeItem(i18n("Edit question and/or answer and click correct."), 0);
			fcbutton->setText(i18n("Correct"));
			question->setText(QString::fromLocal8Bit(element->q));
			answer->setText(QString::fromLocal8Bit(element->a));
		}
		else
		{
			KMessageBox::sorry(this, i18n("The text you entered was not found in the repetition database."));
		}
	}
}

/** Setting the Font for the input-fields */
void KQADialog::setFonts(const QFont& q, const QFont& a)
{
	question->setFont(q);
	answer->setFont(a);
}
